from examples.utils.prepare_env import get_api_key
from tonplay.methods import TonPlayApi

client = TonPlayApi(get_api_key())
print(
    client.get_user_by_identifier("01cf820f-b173-327f-a0e4-3e72feb1c4b8")
)

print(
    client.get_users_by_identifiers([
        "d9931e3566e3b0e2fd846def9bfde53f7f6416c314d85fc9c98c69d04aa37275",
        "6b5283ab-d100-3c5b-8ad7-091c08282e70"
    ])
)



import json
import logging
from json import JSONDecodeError

import requests

from .__version__ import __version__
from .error import ClientError, ServerError, ParameterArgumentError


class Api(object):
    """API base class
        Keyword Args:
            base_url (str, optional): the API base url. By default, it's https://external.api.tonplay.io/x
    """

    def _validate_api_key(self, api_key):
        if len(api_key) != 31:
            raise ParameterArgumentError("wrong api_key length")
        decomposed = api_key.split(":")
        if len(decomposed) != 2:
            raise ParameterArgumentError("wrong api_key format")

    def __init__(
            self,
            api_key=None,
            base_url=None
    ):
        self._validate_api_key(api_key)
        self.api_key = api_key
        self.base_url = base_url

        self.session = requests.Session()
        self.session.headers.update(
            {
                "Content-Type": "application/json",
                "User-Agent": "tonplay-connector/" + __version__,
                "X-Auth-Tonplay": api_key,
            }
        )
        self._logger = logging.getLogger(__name__)
        self._game_key = api_key.split(":")[0]

        return

    def query(self, url_path, **kwargs):
        return self.send_request("GET", url_path, **kwargs)

    def send_request(self, http_method, url_path, payload=None, **kwargs):
        if payload is None:
            payload = {}

        url = self.base_url + url_path
        self._logger.debug("url: " + url)
        params = {
            "url": url,
            "params": kwargs
        }
        if http_method != "GET":
            params["data"] = json.dumps(payload)

        response = self._dispatch_request(http_method)(**params)
        self._logger.debug("raw response from server:" + response.text)
        self._handle_exception(response)

        try:
            data = response.json()
        except ValueError:
            data = response.text
        result = {}

        if len(result) != 0:
            result["data"] = data
            return result

        return data

    def _dispatch_request(self, http_method):
        return {
            "GET": self.session.get,
            "DELETE": self.session.delete,
            "PUT": self.session.put,
            "POST": self.session.post,
        }.get(http_method, "GET")

    def _handle_exception(self, response):
        status_code = response.status_code
        # print(response.text)
        if status_code < 400:
            return
        if 400 <= status_code < 500:
            try:
                err = json.loads(response.text)
            except JSONDecodeError:
                raise ClientError(
                    status_code, None, response.text, None, response.headers
                )
            error_data = None
            if "timestamp" in err:
                error_data = {
                    "type": err["error"],
                    "message": err["message"]
                }
            else:
                error_data = err["error"]
            raise ClientError(
                status_code, error_data["type"], error_data["message"], response.headers
            )
        raise ServerError(status_code, response.text)



from tonplay.api import Api


class TonPlayApi(Api):
    def __init__(self, api_key, **kwargs):
        if "base_url" not in kwargs:
            kwargs["base_url"] = "https://external.api.tonplay.io/x"
        super().__init__(api_key, **kwargs)

    # Market

    from tonplay.methods._market import get_assets_on_sale
    from tonplay.methods._market import get_sft_market_depth
    from tonplay.methods._market import put_on_sale
    from tonplay.methods._market import remove_from_sale
    from tonplay.methods._market import buy_asset

    # TON

    from tonplay.methods._ton import get_new_sft_tokens_mint_link
    from tonplay.methods._ton import get_nft_owner
    from tonplay.methods._ton import get_address_info
    from tonplay.methods._ton import get_burn_asset_link
    from tonplay.methods._ton import get_asset_transfer_link
    from tonplay.methods._ton import get_collectable_item_mint_link
    from tonplay.methods._ton import get_collection_deploy_link
    from tonplay.methods._ton import get_single_item_deploy_link

    # Games

    from tonplay.methods._game import make_withdraw_request

    # Auth

    from tonplay.methods._auth import check_tonplay_token

    # Assets

    from tonplay.methods._asset import get_user_assets

    # Users

    from tonplay.methods._user import get_user_by_identifier
    from tonplay.methods._user import get_users_by_identifiers



from tonplay.lib.enums import AssetType
from tonplay.lib.utils import check_enum_parameter, check_ton_address


def get_assets_on_sale(self):
    """
    Get assets on sale by gameKey

    GET /market/v1/game/{gameKey}
    """

    url_path = f"/market/v1/game/{self._game_key}"
    return self.query(url_path)


def get_sft_market_depth(self, address):
    """
    Get same SFT on sale by asset address

    GET /market/v1/item/{address}

    Args:
        address (str): sft asset address

    """
    url_path = f"/market/v1/item/{address}"
    return self.query(url_path)


def put_on_sale(self, address, price, amount, type, seller_address):
    """
    Put asset on sale

    PUT /market/v1/sale

    Args:
        address (str)       : asset address
        price (float)       : the price in TON
        amount (int)        : amount of assets for sale ( 1 by default)
        type (str)          : AssetType ( NFT | SFT )
        seller_address (str) : seller address (asset's owner address)

    """
    check_enum_parameter(type, AssetType)
    check_ton_address(address, "address")
    check_ton_address(seller_address, "seller_address")

    url_path = f"/market/v1/sale"
    payload = {"address": address, "price": price, "amount": amount, "type": type, "sellerAddress": seller_address}
    return self.send_request("PUT", url_path, payload=payload)


def remove_from_sale(self, address, amount, type, seller_address):
    """
        Remove asset from sale

        DELETE /market/v1/sale

        Args:
            address (str)       : asset address
            amount (int)        : amount of assets for removing from sale ( 1 by default)
            type (str)          : AssetType ( NFT | SFT )
            seller_address (str) : seller address (asset's owner address)

        """
    check_enum_parameter(type, AssetType)
    check_ton_address(address, "address")
    check_ton_address(seller_address, "seller_address")

    url_path = f"/market/v1/sale"
    payload = {"address": address, "amount": amount, "type": type, "sellerAddress": seller_address}
    return self.send_request("DELETE", url_path, payload=payload)


def buy_asset(self, address, amount, type, seller_address, buyer_address):
    """
        Buy asset

        POST /market/v1/sale

        Args:
            address (str)       : asset address
            amount (int)        : amount of assets to buy ( 1 by default)
            type (str)          : AssetType ( NFT | SFT )
            seller_address (str) : seller address (asset's owner address)
            buyer_address (str) : buyer address

    """
    check_enum_parameter(type, AssetType)
    check_ton_address(address, "address")
    check_ton_address(seller_address, "seller_address")
    check_ton_address(seller_address, "buyer_address")

    url_path = f"/market/v1/sale"
    payload = {"address": address, "amount": amount, "type": type, "sellerAddress": seller_address, "buyerAddress": buyer_address}
    return self.send_request("POST", url_path, payload=payload)



from examples.utils.prepare_env import get_api_key
from tonplay.methods import TonPlayApi

client = TonPlayApi(get_api_key())

print(
    client.get_nft_owner("EQCWxV31UgGZZcnuKoGYnY6LlxE7stVZ1kXIRaML1N3fIHgF")
)

print(
    client.get_address_info("EQCWxV31UgGZZcnuKoGYnY6LlxE7stVZ1kXIRaML1N3fIHgF")
)

print(
    client.get_burn_asset_link(
        # asset to burn
        address="",
        # asset type
        type="NFT",
        # amount (default=1)
        amount=None,
        # current owner address
        owner_address=""
    )
)

print(
    client.get_asset_transfer_link(
        # asset to transfer
        address="",
        # asset type
        type="NFT",
        # amount (default=1)
        amount=None,
        # current owner address
        current_owner="",
        # new owner address
        new_owner=""
    )
)



from examples.utils.prepare_env import get_api_key
from tonplay.methods import TonPlayApi

client = TonPlayApi(api_key=get_api_key())

print(
    client.get_assets_on_sale()
)

print(
    client.get_sft_market_depth("EQA4R4qC__ogFwb_27vYQ4Y8Lyg_N4K4Rb2RaGWxnwj2x4Ya")
)

print(
    client.put_on_sale(
        # asset address
        address="",
        # price in TON
        price=10,
        # amount (for SFT) by default = 1
        amount=1,
        # type
        type='SFT',
        # seller address = current owner address
        seller_address=""
    )
)

print(
    client.remove_from_sale(
        # asset address
        address="",
        # amount for remove (for SFT) by default = 1
        amount=1,
        # type
        type='SFT',
        # seller address = current owner address
        seller_address=""
    )
)

print(
    client.buy_asset(
        # asset address
        address="",
        # amount for remove (for SFT) by default = 1
        amount=1,
        # type
        type='SFT',
        # seller address = who is selling
        seller_address="",
        # buyer address = who is buying right now
        buyer_address=""
    )
)




def check_tonplay_token(self, token):
    """
        Verify TONPLAY JWT token

        POST /auth/v1/check

        Args:
            token (str) : jwt token
    """
    url_path = "/auth/v1/check"
    return self.send_request("POST", url_path, token=token)



import uuid

from examples.utils.prepare_env import get_api_key
from tonplay.methods import TonPlayApi

client = TonPlayApi(api_key=get_api_key())

withdrawal_id = str(uuid.uuid4())
print(
    client.make_withdraw_request(withdrawal_id)
)




def make_withdraw_request(self, withdrawal_id):
    """
        Make payout request

        POST /gamecenter/v1/withdraw

        How it works:

        Your project has own built-in wallet inside TONPLAY.
        Your project need to have 'payout-verify-endpoint' inside TONPLAY platform that must return
        { amount: float, address: str } by requested payout_id

        You can make auto-withdraw from built-in wallet over API request by next logic:

        - create withdraw request in your database per each user
        - pass ID of this request into our API (this method)
        - TONPLAY will make verify request on your verify URL for check is this valid withdraw or not
        - if your API answer is OK than TONPLAY will send requested TON amount to selected address

        Args:
            withdrawal_id (str) : withdrawal_id from your system in any format
    """

    url_path = f"/gamecenter/v1/withdraw"
    payload = {"withdrawalId": withdrawal_id}
    return self.send_request("POST", url_path, payload=payload)



from examples.utils.prepare_env import get_api_key
from tonplay.methods import TonPlayApi

client = TonPlayApi(get_api_key())
tonplay_jwt_token = """
eyJhbGciOiJSUzI1NiJ9.eyJzdWIiOiJjY2MyYTJhNDg5NTRmNzQwMGNkNjFmMTc2MjNkZWM3YTE5MzcyNjA5ZGEyZjcwMWMyNzFjM2FmZDg1NWQ4MjkwIiwiaXNUZXN0Ijp0cnVlLCJpc3MiOiJodHRwczpcL1wvb2EudG9ucGxheS5pbyIsImV4cCI6MTY4ODQxMjY4OCwidG9rZW5fdHlwZSI6IkFDVElWRSIsImlhdCI6MTY3OTc3MjY4OCwidGVhbV9rZXkiOiI4cVNlcmthQ3RYIiwiYXV0aG9yaXRpZXMiOlsiUk9MRV9VU0VSIiwiUk9MRV9URVNUX1VTRVIiLCJST0xFX0RFVkVMT1BFUiJdLCJ1c2VybmFtZSI6InNlbnNodV84MDA2Mjk5NiJ9.LrI-K-f1zIQCUEGof6abcIaBvNjr016KE2ciwM_PJ4mhhhxtVOY4NDESgip5zVV-p34MXdY9KsqO4prbSEggJzxrpgpAR3aGENXOFDj7p54t_ctt5BlJnmE802qwKipEVfNC3I6PoUtr6512CO8snULZRytHGO2b5q2Jp-Ap3ZaRErkRmqNj7ocrEGjrNbIRi-b-LjTfaDVlvIzq4RHq2ZHWjdfjHA5IRy-rAmxeVlzHfsWzx5-d2UIcXuMhsl17mtC0yQQlf0sNd0iZSThWl-Y6kB3OH9Th3QCC5cHzasYzyfLSDQaz-gu-DytPE12f3N805HIGnqFbMJzvB8OJUQ
"""

print(
    client.check_tonplay_token(tonplay_jwt_token)
)




def get_user_by_identifier(self, identifier):
    f"""
    Get info about TONPLAY user by identifier

    GET /auth/v1/user/identifier
    
    Args:
        identifier (str) : unique user id on TONPLAY
    """
    url_path = f"/auth/v1/user/{identifier}"
    return self.query(url_path)


def get_users_by_identifiers(self, identifiers_list):
    """
    Get info about TONPLAY users by their identifiers

    GET /auth/v1/user

    Args:
        identifiers_list [str] : list of unique users ids on TONPLAY

    """
    url_path = f"/auth/v1/user?identifiers=1"
    return self.query(url_path, identifiers=identifiers_list)



import os

from setuptools import setup, find_packages

NAME = "tonplay-sdk"
VERSION = None
DESCRIPTION = (
    "This is a lightweight library that works as a connector to TON Play public API"
)

about = {}
root = os.path.abspath(os.path.dirname(__file__))
with open(
    os.path.join(os.path.dirname(__file__), "requirements/common.txt"), "r"
) as fh:
    requirements = fh.readlines()

with open("README.md", "r") as fh:
    about["long_description"] = fh.read()

if not VERSION:
    with open(os.path.join(root, "tonplay", "__version__.py")) as f:
        exec(f.read(), about)
else:
    about["__version__"] = VERSION

setup(
    name=NAME,
    version=about["__version__"],
    description=DESCRIPTION,
    long_description=about["long_description"],
    long_description_content_type="text/markdown",
    license='MIT',
    author="TON Play",
    packages=find_packages(exclude=("tests",)),
    keywords=["TON Play", "Ton Play API", "Tonplay API", "ton sdk", "ton"],
    install_requires=[req for req in requirements],
    python_requires=">=3.11.1",
)



from tonplay.lib.enums import AssetType
from tonplay.lib.utils import check_ton_address, check_enum_parameter


def get_nft_owner(self, address):
    """
        Get NFT owner's address

        GET /tondata/v1/ton/{address}/owner

        Args:
            address (str): NFT address
        """
    check_ton_address(address, "address")
    url_path = f"/tondata/v1/ton/{address}/owner"
    return self.query(url_path)


def get_address_info(self, address):
    """
        Get information about contract

        GET /tondata/v1/ton/{address}/status

        Args:
            address (str): contract address
        """
    check_ton_address(address, "address")
    url_path = f"/tondata/v1/ton/{address}/status"
    return self.query(url_path)


def get_burn_asset_link(self, address, amount, type, owner_address):
    """
        Burn selected asset

        DELETE /tondata/v1/ton/burn/{address}

        Args:
            address (str)   : asset address to burn
            type (str)      : AssetType ( NFT | SFT )
            amount (int)    : asset amount to burn (for SFT) default 1
            owner_address   : asset's owner
    """
    check_ton_address(address, "address")
    check_enum_parameter(type, AssetType)
    url_path = f"/tondata/v1/ton/burn/{address}"
    payload = {"amount": amount, "ownerAddress": owner_address, "type": type}
    return self.send_request("DELETE", url_path, payload=payload)


def get_asset_transfer_link(self, address, amount, type, current_owner, new_owner):
    """
        Generate link to transfer asset

        POST /tondata/v1/ton/transfer/{address}

        Args:
            address (str)   : asset address to burn
            type (str)      : AssetType ( NFT | SFT )
            amount (int)    : asset amount to burn (for SFT) default 1
            current_owner   : current asset's owner
            new_owner       : new asset's owner

    """
    check_ton_address(address, "address")
    check_ton_address(current_owner, "current_owner")
    check_ton_address(new_owner, "new_owner")
    check_enum_parameter(type, AssetType)
    url_path = f"/tondata/v1/ton/transfer/{address}"
    payload = {"amount": amount, "newOwner": new_owner, "currentOwnerAddress": current_owner}
    return self.send_request("POST", url_path, payload=payload, type=type)


def get_collection_deploy_link(
        self,
        type,
        owner,
        metadata_url,
        common_metadata_url,
        royalty_percent,
        royalty_beneficiary_address
):
    """
        Generate link to deploy collection

        POST /tondata/v1/ton/deploy/collection

        Args:
            type (str)                          : AssetType ( NFT | SFT )
            owner (str)                         : who will be asset's owner
            metadata_url (str)                  : collection metadata url
            common_metadata_url (str)           : common item medata url
            royalty_percent (float)             : royalty percent (e.g. 1.5 , 2 , 4.5 )
            royalty_beneficiary_address (str)   : royalty recipient address

    """
    check_ton_address(owner, "owner")
    check_ton_address(royalty_beneficiary_address, "royalty_beneficiary_address")
    check_enum_parameter(type, AssetType)
    url_path = f"/tondata/v1/ton/deploy/collection"

    denominator = pow(10, len(str(float(royalty_percent)).split(".")[1]))
    numerator = royalty_percent * denominator
    payload = {
        "owner": owner,
        "maxSupply": 0,
        "collectionMetadataUrl": metadata_url,
        "itemMetadataCommonUrl": common_metadata_url,
        "royalty": {
            "numerator": numerator,
            "denominator": denominator,
            "beneficiaryAddress": royalty_beneficiary_address
        }
    }
    return self.send_request("POST", url_path, payload=payload, type=type)


def get_single_item_deploy_link(
        self,
        type,
        owner,
        metadata_url
):
    """
        Generate link to deploy single asset

        POST /tondata/v1/ton/deploy/single

        Args:
            type (str)                          : AssetType ( NFT | SFT )
            owner (str)                         : who will be asset's owner
            metadata_url (str)                  : collection metadata url

    """
    check_ton_address(owner, "owner")
    check_enum_parameter(type, AssetType)
    url_path = f"/tondata/v1/ton/deploy/single"
    payload = {
        "owner": owner,
        "maxSupply": 0,
        "metadata": metadata_url,
    }
    return self.send_request("POST", url_path, payload=payload, type=type)


def get_collectable_item_mint_link(
        self,
        address,
        type,
        owner,
        item_metadata_url,
        amount
):
    """
        Generate link to mint new collectable item

        POST /tondata/v1/ton/mint/collection/{address}

        Args:
            type (str)                          : AssetType ( NFT | SFT )
            address (str)                       : collection address
            owner (str)                         : who will be asset's owner
            item_metadata_url (str)             : item metadata part url
            amount (int)                        : how many collectables need to be deployed

    """
    check_ton_address(owner, "owner")
    check_ton_address(address, "address")
    check_enum_parameter(type, AssetType)
    url_path = f"/tondata/v1/ton/mint/collection/{address}"
    payload = {
        "owner": owner,
        "itemMetadataPartOfUrl": item_metadata_url,
        "amount": amount,
    }
    return self.send_request("POST", url_path, payload=payload, type=type)


def get_new_sft_tokens_mint_link(
        self,
        address,
        owner,
        amount
):
    """
        Generate link to mint new SFT tokens

        POST /tondata/v1/ton/mint/sft/{address}

        Args:
            address (str)                       : collection address
            owner (str)                         : who will be asset's owner
            amount (int)                        : how many collectables need to be deployed

    """
    check_ton_address(owner, "owner")
    check_ton_address(address, "address")
    url_path = f"/tondata/v1/ton/mint/sft/{address}"
    payload = {
        "owner": owner,
        "amount": amount,
    }
    return self.send_request("POST", url_path, payload=payload)



import base64

from tonplay.error import ParameterValueError


def check_enum_parameter(value, enum_class):
    if value not in set(item.name for item in enum_class):
        raise ParameterValueError([value])


bounceable_tag, non_bounceable_tag = b'\x11', b'\x51'
b64_abc = set('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890+/')
b64_abc_urlsafe = set('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-')


def check_ton_address(address, argument_name):
    if set(address).issubset(b64_abc):
        address_bytes = base64.b64decode(address.encode('utf8'))
    elif set(address).issubset(b64_abc_urlsafe):
        address_bytes = base64.urlsafe_b64decode(address.encode('utf8'))
    else:
        raise Exception(f"{argument_name} Not an address")
    if not calcCRC(address_bytes[:-2]) == address_bytes[-2:]:
        raise Exception(f"{argument_name} Wrong checksum")


def calcCRC(message):
    poly = 0x1021
    reg = 0
    message += b'\x00\x00'
    for byte in message:
        mask = 0x80
        while(mask > 0):
            reg <<= 1
            if byte & mask:
                reg += 1
            mask >>= 1
            if reg > 0xffff:
                reg &= 0xffff
                reg ^= poly
    return reg.to_bytes(2, "big")



from examples.utils.prepare_env import get_api_key
from tonplay.methods import TonPlayApi

client = TonPlayApi(get_api_key())

print(
    client.get_user_assets("EQBo5Rdhjl02oc1rAzwu_qxAWM6WCwxxUJckokyWEaYXG3xl")
)



from examples.utils.prepare_env import get_api_key
from tonplay.methods import TonPlayApi

client = TonPlayApi(get_api_key())

# DEPLOY API

print(
    client.get_collection_deploy_link(
        # collection type (NFT | SFT)
        type="NFT",
        # collection owner address
        owner="",
        # collection metadata json url
        # https://docs.tonplay.io/digital-assets-api/metadata/assets-collection
        metadata_url="",
        # items common metadata url
        # for example: https://examplegame:5000/nfts
        common_metadata_url="",
        # royalty percent
        # for example: 1 (1%) , 2.5 (2.5%) etc
        royalty_percent=3.5,
        # royalty beneficiary address who will receive royalty
        royalty_beneficiary_address=""
    )
)

# mint new item from collection
print(
    client.get_collectable_item_mint_link(
        # collection address
        address="",
        # collection type (NFT | SFT)
        type="NFT",
        # new item owner address
        owner="",
        # last part of item metadata url
        # usually is filename.json
        # so finally collectionComonUrl + itemUrl = full link to item metadata
        item_metadata_url="10.json",
        # how many items will be mint
        amount=2
    )
)

# mint new SFT tokens
print(
    client.get_new_sft_tokens_mint_link(
        # SFT master contract (minter) address
        address="",
        # owner address of new minted tokens
        owner="",
        # how many tokens will be deployed (int)
        amount=100
    )
)

print(
    client.get_single_item_deploy_link(
        # asset type (NFT | SFT)
        type="NFT",
        # asset owner address
        owner="",
        # metadata json url
        # https://docs.tonplay.io/digital-assets-api/metadata/nft-assets
        metadata_url=""
    )
)




def get_user_assets(self, user_address):
    """
        Get assets belongs to user account

        GET /tondata/v2/assets/{address}

        Args:
            user_address (str) : user TON address

    """
    url_path = f"/tondata/v2/assets/{user_address}"
    return self.query(url_path)
